% ED of AA triangular lattice, 2 flavors, 2*2-orbitals per site
clear;
%close all;
N=4;   % number of sites, 4 sites periodic. Calculate the spectral weight of the 1st site
Nq=4*N;  % number of qubits

% for N=4 periodic
nnb=2*(ones(N,N)-eye(N)); % nearest neighbour
%nnb(1,2)=2; nnb(2,1)=2;
%nnb(3,4)=2; nnb(4,3)=2;
%nnb(5,6)=2; nnb(6,5)=2;

nnbb=kron(eye(4),nnb);   % nnb with flavors and bands

D0=0;
disE=D0*randn(2,N);    % random potential

C0=15;   % charging energy meV, adding an energy C0*(n-f)^2/2, f: filling per site, n: electron per site

E0=8; %6;   % upper/lower flat band energies at -E0 and +E0.
t=0.6;%0.5;   % nearest hopping within the same band

U=1*[23,23]; % on-site interaction between the [same band, different band]
%U=U+C0*[1,1];  % modified by charging energy
U0=U(1); U1=U(2);
V=[0,0];  % nearest-site interaction between [same band, different band]
V0=V(1);V1=V(2);

%----label all the states with binary numbers. Basis: CBf1, VBf1, CBf2, VBf2
WN=de2bi(0:2^Nq-1);   % size WN:  2^Nq * Nq
WWN=[sum(WN(:,1:N),2),sum(WN(:,N+1:2*N),2),sum(WN(:,2*N+1:3*N),2),sum(WN(:,3*N+1:4*N),2)];  % number in 4 basis

%----interaction & on-site matrix in the number basis
HI=zeros(Nq,Nq);
for j=0:3
    HI(j*N+1:(j+1)*N,j*N+1:(j+1)*N)=(-1)^j*E0*eye(N)+diag(disE(1+mod(j+1,2),:))+V0*nnb/2;
    for jj=j+1:3
        HI(j*N+1:(j+1)*N,jj*N+1:(jj+1)*N)=U(1+mod(j+jj,2))*eye(N)/2+V(1+mod(j+jj,2))*nnb/2;
        HI(jj*N+1:(jj+1)*N,j*N+1:(j+1)*N)=HI(j*N+1:(j+1)*N,jj*N+1:(jj+1)*N);
    end
end
EI=sum((WN*HI).*WN,2);  % on site energy

%----number of electron configurations, 2 flavors 2 bands
NN=zeros((N+1)^4,4);
n1=0:N; n1=n1';
for j1=1:4
    NN(:,j1)=kron(ones((N+1)^(j1-1),1),kron(n1,ones((N+1)^(4-j1),1)));
end
[Ntot,I]=sort(sum(NN,2));    % sort in order of total electrons
NN=NN(I,:);

Eg=Inf*ones(Nq+1,1);    % ground state energy for each filling
Ng=zeros(Nq+1,4);    % number density of GS
Qg=zeros(nchoosek(N,floor(N/2))^4,Nq+1);  % ground state wave function
IN=zeros(nchoosek(N,floor(N/2))^4,Nq+1);  % basis indices

PN=zeros(Nq+1,1);   % partition number of Ne electrons into Nq qubits
jm=0;
Emax=10000;
for Ne=0:Nq      % number of electrons
    Dh=sum(Ntot==Ne);
    PN(Ne+1)=Dh;   
    for j=1:Dh
        jm=jm+1;
        N4=NN(jm,:);
        Isub=find(sum(WWN==N4,2)==4); % index of basis
        Nb=length(Isub);        % Hamiltonian size
        Wb=WN(Isub,:);          % subbasis
        Hb=diag(EI(Isub));         % subHamiltonian
        for j1=1:Nb
            for j2=j1+1:Nb
                Df=mod(Wb(j1,:)-Wb(j2,:),2);
                if sum(Df)==2
                    nf=find(Df);
                    Hb(j1,j2)=t*nnbb(nf(1),nf(2))*(-1)^(sum(Wb(j1,nf(1)+1:nf(2)-1)));  % JW trans
                    Hb(j2,j1)=Hb(j1,j2)';
                end
            end
        end
        Hb=Hb-eye(Nb)*C0*(Ne-Ne^2/N)/2;
        [Q0,Eb]=eigs(Emax*eye(Nb)-Hb,1);   % GS of Hb
        Eb=Emax-Eb;                        % GS energy
        if Eg(Ne+1)>Eb
            Eg(Ne+1)=Eb;     % sub ground state energy
            Ng(Ne+1,:)=N4;      % GS configuration
            Qg(1:Nb,Ne+1)=Q0;   % GS wave function
            IN(1:Nb,Ne+1)=Isub; % basis index
        end
    end
end

Ef=zeros(Nq+1,1);%------chemical potential----------

Nbb=600;
bb=(-Nbb:Nbb)*150/Nbb;  % energy range of bias
eta=2.5;  % broadening of delta function
LDOS=zeros(Nq+1,length(bb));

%-------Spectral Weight---------
for Ne=0:Nq
    a1=max(Ne,1);a2=min(Ne+2,Nq+1);
    Ef(Ne+1)=(Eg(a2)-Eg(a1))/(a2-a1);  % chemical potential
    
    N4=Ng(Ne+1,:);
    Igs=nonzeros(IN(:,Ne+1));
    Qgs=Qg(1:length(Igs),Ne+1);
    Wbb=WN(Igs,:);   % basis of GS
    for j=1:4
        if N4(j)<N   % electron side
            dN=zeros(1,Nq);
            dN((j-1)*N+1)=1;
            N4s=N4; N4s(j)=N4s(j)+1;
            Isub=find(sum(WWN==N4s,2)==4); % index of basis
            Nb=length(Isub);        % Hamiltonian size
            Wb=WN(Isub,:);          % subbasis
            Hb=diag(EI(Isub));         % subHamiltonian
            A1=zeros(length(Igs),Nb);  % annihilation matrix on site 1
            for j1=1:Nb
                for j2=j1+1:Nb
                    Df=mod(Wb(j1,:)-Wb(j2,:),2);
                    if sum(Df)==2
                        nf=find(Df);
                        Hb(j1,j2)=t*nnbb(nf(1),nf(2))*(-1)^(sum(Wb(j1,nf(1)+1:nf(2)-1)));  % JW trans
                        Hb(j2,j1)=Hb(j1,j2)';
                    end
                end
                for j2=1:length(Igs)
                    if sum((Wb(j1,:)-Wbb(j2,:))==dN)==Nq
                        A1(j2,j1)=(-1)^(sum(Wb(j1,1:(j-1)*N)));
                    end
                end
            end
            Hb=Hb-eye(Nb)*C0*((Ne+1)-(Ne+1)^2/N)/2;
            [Q0,Eb]=eig(Hb);
            Eb=real(diag(Eb))-Eg(Ne+1)-Ef(Ne+1)+1i*eta;   % energy levels with imag
            eM=abs(Qgs'*A1*Q0).^2;    %    matrix elements squared
            [X,Y]=meshgrid(bb,Eb);
            LDOS(Ne+1,:)=LDOS(Ne+1,:)+sum(imag((X-Y).^(-1)).*eM',1);
        end
        
        if N4(j)>0            % hole side
            dN=zeros(1,Nq);
            dN((j-1)*N+1)=1;
            N4s=N4; N4s(j)=N4s(j)-1;
            Isub=find(sum(WWN==N4s,2)==4); % index of basis
            Nb=length(Isub);        % Hamiltonian size
            Wb=WN(Isub,:);          % subbasis
            Hb=diag(EI(Isub));         % subHamiltonian
            A1=zeros(length(Igs),Nb);  % creation matrix
            for j1=1:Nb
                for j2=j1+1:Nb
                    Df=mod(Wb(j1,:)-Wb(j2,:),2);
                    if sum(Df)==2
                        nf=find(Df);
                        Hb(j1,j2)=t*nnbb(nf(1),nf(2))*(-1)^(sum(Wb(j1,nf(1)+1:nf(2)-1)));  % JW trans
                        Hb(j2,j1)=Hb(j1,j2)';
                    end
                end
                for j2=1:length(Igs)
                    if sum((Wbb(j2,:)-Wb(j1,:))==dN)==Nq
                        A1(j2,j1)=(-1)^(sum(Wb(j1,1:(j-1)*N)));
                    end
                end
            end
            Hb=Hb-eye(Nb)*C0*((Ne-1)-(Ne-1)^2/N)/2;
            [Q0,Eb]=eig(Hb);
            Eb=-real(diag(Eb))+Eg(Ne+1)-Ef(Ne+1)+1i*eta;   % energy levels with imag
            eM=abs(Qgs'*A1*Q0).^2;    %    matrix elements squared
            [X,Y]=meshgrid(bb,Eb);
            LDOS(Ne+1,:)=LDOS(Ne+1,:)+sum(imag((X-Y).^(-1)).*eM',1);
        end
    end
end
FDOS=LDOS(:,Nbb+1);     % Fermi energy DOS

ffh=(0:Nq)*2/Nq-1;  % filling fraction

colorset=varycolor(Nq+1);  % 'Color',colorset(Ne+1,:)
figure;
hold on;
f0=0.24; %0.12
for Ne=0:Nq
    plot(bb,f0*LDOS(Ne+1,:)+ffh(Ne+1),'k');
end
s=15;   % slope in higher bands
for j=1:3
    plot(bb+s*j,f0*LDOS(1,:)+ffh(1)-2*j/Nq,'k');
    plot(bb-s*j,f0*LDOS(Nq+1,:)+ffh(Nq+1)+2*j/Nq,'k');
end
xlabel('Bias(meV)');
ylabel('filling');
xlim([-70,70]);
ylim([-1.5,1.6]);

shif=[2,15,15,4.5]; % shift of integration of the Fermi level DOS
for j=0:3
    Ft=zeros(N+1,1);
    F=FDOS(j*N+1:(j+1)*N+1);
    F=F.^-1;
    for jj=1:N+1
        if j<2
            Ft(jj)=-sum(F(1:jj))+(F(1)+F(jj))/2+shif(j+1);
        else
            Ft(jj)=sum(F(jj:N+1))-(F(N+1)+F(jj))/2-shif(j+1);
        end
    end
    %plot(Ft,ffh(j*N+1:(j+1)*N+1),'LineWidth',1);
end

figure;
plot(FDOS,ffh,'k');
xlabel('DOS at Fermi energy');
ylabel('filling');

figure;    % Fermi level change
plot(ffh,Ef);
xlabel('filling');
ylabel('chemical potential (meV)');

%figure;    % Fermi level change
%plot(ffh,Eg);
%xlabel('filling');
%ylabel('ground state energy (meV)');

% ---chemical potential jump (blue line) & distance between tailfeathers (red line) at 0.5 filling---
% ---this data is for E0=8, t=0.6 fixed, while U is varied
Uh=[0,0.125,0.25,0.375,0.5,0.625,0.75,0.875,1,1.125,1.25,1.375,1.5]*25;
dEf=[0,1.72,4.11,6.82,9.70,12.67,15.68,18.73,21.79,24.87,27.96,31.05,34.15];
dT=[0,0,0,3.75,9.1,15,18.5,21.25,24.5,27.75,31,34,37];
figure; 
hold on; 
plot(Uh,dEf,'LineWidth',1); plot(Uh,dT,'LineWidth',1); plot(Uh,Uh,'k--');
xlabel('U (meV)'), ylabel('E (meV)');
