% ED for one quantum dot with 2 bands, 4 flavors

clear;
close all;

E0=8;             %6;   % upper/lower flat band energies at -E0 and +E0.
U=1*[25,25];      % on-site interaction (meV) between the [same band, different band]
U0=U(1); U1=U(2);

NH=de2bi(0:2^8-1); % number occupations
n1=sum(NH(:,1:4),2);  % electron number in orbital 1
n2=sum(NH(:,5:8),2);  % electron number in orbital 2
En=U0*(n1.*(n1-1)+n2.*(n2-1))/2+U1*n1.*n2+E0*(n2-n1);    % on-site energy level from 0 to 8 electrons
Ht=zeros(2^10,2);    % hopping matrix of adding one electron
jt=0;
for jn=1:2^8
    for jm=jn+1:2^8
        DN=NH(jn,:)-NH(jm,:);
        if norm(DN)==1
            jt=jt+1;
            Ht(jt,:)=[jm,jn];
        end
    end
end

T0=0.5;   % temperature in meV

Nmu=1000;  % number of chemical potential steps
mu=((1:Nmu)-Nmu/5)*12*U0/Nmu;   % chemical potentials
fmu=zeros(1,Nmu);     % filling at mu
Nbias=200;  % bias energy discretization
eta=1;      % peak broadening
Ebias=(-Nbias:Nbias)*3*U0/Nbias;   % bias energy
SW=zeros(Nmu,2*Nbias+1);     % spectral function

Eg=15;   % gap to the remote band
Drm=20;   % compressibility dmu/dn of remote band
mumax=E0+4*U1+3*U0+Eg;  % higher mu reaching remote band
mumin=-E0-Eg;  % lower mu reaching remote band

for j=1:Nmu
    muj=mu(j);
    Ej=En-muj*(n1+n2);
    Ej=Ej-min(Ej);    % reset GS energy to zero
    Wj=exp(-Ej/T0);   % Boltzmann weight
    Zj=sum(Wj);  % partition function
    fmu(j)=sum(Wj.*(n1+n2-4))/Zj;   % filling
    if muj>mumax
        fmu(j)=fmu(j)+(muj-mumax)/Drm;
    end
    if muj<mumin
        fmu(j)=fmu(j)+(muj-mumin)/Drm;
    end
    for jj=1:jt
        SW(j,:)=SW(j,:)+(((Ebias-Ej(Ht(jj,1))+Ej(Ht(jj,2))).^2+eta^2).^-1)*(Wj(Ht(jj,1))+Wj(Ht(jj,2)))*eta/Zj;
    end
end

%----smearing out---------
df=0.005;
for jn=-4:4
    [a,N1]=min(abs(fmu-jn+df));
    [b,N2]=min(abs(fmu-jn-df));
    SWW=SW(N1:N2,:);
    SW(N1:N2,:)=ones(N2-N1+1,1)*max(SWW);
end
figure;
plot(fmu,mu,'k');
xlabel('filling n');
ylabel('chemical potential \mu');
xlim([-5,5]);

[X,Y]=meshgrid(Ebias,fmu);
figure;
xlabel('Bias E(meV)');
ylabel('filling n');
surface(X,Y,SW,'EdgeColor','none');
view(0,90);
ylim([-5,5]);
xlim([-60,60]);
